@@ -8,8 +8,9 @@ import os
 from pywapor.general.logger import log
 from pywapor.general.performance import performance_check
 from pywapor.general.processing_functions import process_ds
-from pywapor.general.processing_functions import open_ds, save_ds
-from rasterio import CRS
+from pywapor.general.processing_functions import open_ds, save_ds, remove_ds, log_example_ds
+import rasterio
+import warnings
 
 def get_pixel_sizes(dss):
     # Check CRSs of datasets.
@@ -19,8 +20,29 @@ def get_pixel_sizes(dss):
     # Pick the dominant CRS.
     crs = uniqs[np.argmax(counts)]
     # Reproject to common CRS.
-    dss = [ds.rio.reproject(CRS.from_epsg(crs)) for ds in dss]
-    return [np.abs(np.prod(v.rio.resolution())) for v in dss]
+    dss2 = [(ds, ds.rio.reproject(rasterio.CRS.from_epsg(crs))) for ds in dss]
+    return {np.abs(np.prod(ds2.rio.resolution())): ds for ds, ds2 in dss2}
+
+def has_changed(ds):
+    if "source" in ds.encoding.keys():
+        changed = not ds.equals(open_ds(ds.encoding["source"]))
+    else:
+        changed = True
+    return changed
+
+def has_geotransform(ds):
+    varis = ds.data_vars
+    if not "source" in ds.encoding.keys():
+        return False
+    for var in varis:
+        with warnings.catch_warnings(record=True) as w:
+            _ = rasterio.open(f"netcdf:{ds.encoding['source']}:{var}")
+            if len(w) > 0:
+                for warning in w:
+                    no_geot = "Dataset has no geotransform, gcps, or rpcs." in str(warning.message)
+                    if no_geot:
+                        return False
+    return True
 
 def align_pixels(dss, folder, spatial_interp = "nearest", example_ds = None, stack_dim = "time", fn_append = ""):
 
@@ -34,18 +56,31 @@ def align_pixels(dss, folder, spatial_interp = "nearest", example_ds = None, sta
         dss1 = [dss[0]]
     else:
         if isinstance(example_ds, type(None)):
-            example_ds = dss[np.argmin(get_pixel_sizes(dss))]
+            example_ds = min(get_pixel_sizes(dss).items(), key=lambda x: x[0])[1]
+            log_example_ds(example_ds)
         dss1 = list()
         for i, (spat_interp, ds_part) in enumerate(zip(spatial_interp, dss)):
-            if not ds_part.equals(example_ds):
+            if needs_reprojecting(ds_part, example_ds):
                 var_str = "_".join(ds_part.data_vars)
                 dst_path = os.path.join(folder, f"{var_str}_x{i}{fn_append}.nc")
                 ds_part = reproject(ds_part, example_ds, dst_path, spatial_interp = spat_interp, stack_dim = stack_dim)
-                temp_files.append(ds_part.encoding["source"])
+                temp_files.append(ds_part)
+            else:
+                ds_part = ds_part.assign_coords({"y": example_ds.y, 
+                                                "x": example_ds.x, 
+                                                "spatial_ref": example_ds.spatial_ref})
             dss1.append(ds_part)
 
     return dss1, temp_files
 
+def needs_reprojecting(ds, example_ds):
+    check_crs = ds.rio.crs == example_ds.rio.crs
+    check_ysize = ds.y.size == example_ds.y.size
+    check_xsize = ds.x.size == example_ds.x.size
+    check_y = bool((ds.y == example_ds.y).all().values)
+    check_x = bool((ds.x == example_ds.x).all().values)
+    return not np.all([check_crs, check_ysize, check_xsize, check_x, check_y])
+
 def choose_reprojecter(src_ds, max_bytes = 2e9, min_times = 10, stack_dim = "time"):
 
     if stack_dim in src_ds.dims:
@@ -102,7 +137,7 @@ def reproject_chunk(src_ds, example_ds, dst_path, spatial_interp = "nearest", st
     variables = dict()
     ncs = list()
 
-    if not "source" in src_ds.encoding.keys() or not "grid_mapping" in src_ds.encoding.keys():
+    if has_changed(src_ds) or not has_geotransform(src_ds):
         src_path = src_ds.encoding.get("source", dst_path).replace(".nc", "_fixed.nc")
         new_src_ds = src_ds.sortby("y", ascending = False)
         new_src_ds = new_src_ds.rio.write_transform(new_src_ds.rio.transform(recalc=True))
@@ -138,7 +173,7 @@ def reproject_chunk(src_ds, example_ds, dst_path, spatial_interp = "nearest", st
 
         @performance_check
         def _save_warped_vrt(src_path, var, vrt_options, part_path):
-            with rasterio.open(f'netcdf:{src_path}:{var}') as src:
+            with rasterio.open(f'NETCDF:{src_path}:{var}') as src:
                 with WarpedVRT(src, **vrt_options) as vrt:
                     rio_shutil.copy(vrt, part_path, driver='netcdf', creation_options = {"COMPRESS": "DEFLATE"})
 
@@ -168,16 +203,16 @@ def reproject_chunk(src_ds, example_ds, dst_path, spatial_interp = "nearest", st
     ds = save_ds(ds, dst_path, encoding = "initiate", label = label)
 
     for nc in ncs:
-        os.remove(nc)
+        remove_ds(nc)
 
     return ds
 
-if __name__ == "__main__":
+# if __name__ == "__main__":
 
-    src_ds = open_ds(r"/Users/hmcoerver/Local/test_data/SENTINEL3/SL_2_LST___.nc")
-    example_ds = open_ds(r"/Users/hmcoerver/Local/test_data/SENTINEL2/S2MSI2A.nc")
-    dst_path = r"/Users/hmcoerver/Local/test_data/output_test.nc"
-    spatial_interp = "nearest"
-    var = "lst"
+#     src_ds = open_ds(r"/Users/hmcoerver/Local/test_data/SENTINEL3/SL_2_LST___.nc")
+#     example_ds = open_ds(r"/Users/hmcoerver/Local/test_data/SENTINEL2/S2MSI2A.nc")
+#     dst_path = r"/Users/hmcoerver/Local/test_data/output_test.nc"
+#     spatial_interp = "nearest"
+#     var = "lst"
 
-    ds = reproject_chunk(src_ds, example_ds, dst_path, spatial_interp = "nearest")
+#     ds = reproject_chunk(src_ds, example_ds, dst_path, spatial_interp = "nearest")
